#ifdef __APPLE__
#include <GLUT/glut.h> // Apple's version of GLUT is here
#else
#include <GL/glut.h> // but the standard place for it is here
#endif

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "centipede.h"

//============================================================================
// global variables, used by UI functions to manage state

int winwidth=720, winheight=480; // the pixel dimensions of the window

//============================================================================
// basic drawing and UI functions

// set up some simple default lights to make smooth shading work
void set_up_lighting(void)
{
   glEnable(GL_LIGHTING);
   {
      GLfloat ambient[4]={.3, .3, .3, 1};
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
   }
   {
      GLfloat color[4]={.5, .5, .5, 1};
      GLfloat position[3]={0, 0, 0};
      glLightfv(GL_LIGHT0, GL_DIFFUSE, color);
      glLightfv(GL_LIGHT0, GL_SPECULAR, color);
      glLightfv(GL_LIGHT0, GL_POSITION, position);
      glEnable(GL_LIGHT0);
   }
   {
      GLfloat color[4]={.5, .5, .5, 1};
      GLfloat direction[3]={-1, -1, 0};
      glLightfv(GL_LIGHT1, GL_DIFFUSE, color);
      glLightfv(GL_LIGHT1, GL_SPECULAR, color);
      glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, direction);
      glEnable(GL_LIGHT1);
   }
   {
      GLfloat color[4]={.2, .2, .2, 1};
      GLfloat direction[3]={.5, -1, 0};
      glLightfv(GL_LIGHT2, GL_DIFFUSE, color);
      glLightfv(GL_LIGHT2, GL_SPECULAR, color);
      glLightfv(GL_LIGHT2, GL_SPOT_DIRECTION, direction);
      glEnable(GL_LIGHT2);
   }
}

// establish the projection and model-view transformations
void set_up_view(void)
{
   // you need this command to make sure OpenGL draws to the window correctly
   glViewport(0, 0, (GLsizei)winwidth, (GLsizei)winheight);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   // @@@ here you need to add appropriate commands to set up perspective projection for your scene

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   set_up_lighting(); // setting up the lights here "attaches" the lights to the camera, as opposed to the world
   // @@@ here you need to set up the basic world space to camera space transformation for your scene
}

void reshape(int w, int h)
{
   winwidth=w;
   winheight=h;
   glutPostRedisplay(); // better redraw in case the window changed size
}

void display(void)
{
   // clear the screen including the Z buffer
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   // establish the perspective projection and the world-to-camera model-view transform
   set_up_view();

   // draw your masterpiece!
   draw_centipede();

   // and make it visible on the screen
   glutSwapBuffers();
}

int main(int argc, char **argv)
{
   // set up GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
   glutInitWindowSize(winwidth, winheight);
   glutCreateWindow("cs314 assignment 2");
   glutReshapeFunc(reshape);
   glutDisplayFunc(display);

   // set up OpenGL with the basics we need
   glEnable(GL_DEPTH_TEST);
   glClearColor(0, 0, 0, 0);
   glClearDepth(1);
   glEnable(GL_NORMALIZE); // this is necessary in case you non-uniformly scale drawing primitives

   // and turn over control to GLUT and the callbacks
   glutMainLoop();

   return 0;
}

